@@ -42,6 +42,7 @@ struct _mdi *
 _WM_ParseNewHmi(uint8_t *hmi_data, uint32_t hmi_size) {
     uint32_t hmi_tmp = 0;
     uint8_t *hmi_base = hmi_data;
+    uint8_t *data_end = hmi_data + hmi_size;
     uint32_t data_size;
     uint16_t hmi_bpm = 0;
     uint16_t hmi_division = 0;
@@ -75,6 +76,11 @@ _WM_ParseNewHmi(uint8_t *hmi_data, uint32_t hmi_size) {
     } *note;
 
 
+    if (hmi_size <= 370) {
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "file too short", 0);
+        return NULL;
+    }
+
     if (memcmp(hmi_data, "HMI-MIDISONG061595", 18)) {
         _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_NOT_HMI, NULL, 0);
         return NULL;
@@ -87,6 +93,20 @@ _WM_ParseNewHmi(uint8_t *hmi_data, uint32_t hmi_size) {
 
     hmi_track_cnt = hmi_data[228];
 
+    if (!hmi_track_cnt) {
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(no tracks)", 0);
+        return NULL;
+    }
+    if (!hmi_bpm) {
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, "(bad bpm)", 0);
+        return NULL;
+    }
+
+    if (hmi_size < (370 + (hmi_track_cnt * 17))) {
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "file too short", 0);
+        return NULL;
+    }
+
     hmi_mdi = _WM_initMDI();
 
     _WM_midi_setup_divisions(hmi_mdi, hmi_division);
@@ -111,14 +131,15 @@ _WM_ParseNewHmi(uint8_t *hmi_data, uint32_t hmi_size) {
 
     smallest_delta = 0xffffffff;
 
-    if (hmi_size < (370 + (hmi_track_cnt * 17))) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_NOT_HMI, "file too short", 0);
-        goto _hmi_end;
-    }
-
     hmi_track_offset[0] = *hmi_data; // To keep Xcode happy
 
     for (i = 0; i < hmi_track_cnt; i++) {
+        /* FIXME: better and/or more size checks??? */
+        if (data_end - hmi_data < 4) {
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "file too short", 0);
+            goto _hmi_end;
+        }
+
         hmi_track_offset[i] = *hmi_data++;
         hmi_track_offset[i] += (*hmi_data++ << 8);
         hmi_track_offset[i] += (*hmi_data++ << 16);
@@ -343,7 +364,7 @@ _WM_ParseNewHmi(uint8_t *hmi_data, uint32_t hmi_size) {
 
         _hmi_next_track:
             hmi_tmp = 0;
-            UNUSED(hmi_tmp);
+            WMIDI_UNUSED(hmi_tmp);
         }
 
         // convert smallest delta to samples till next
@@ -358,7 +379,7 @@ _WM_ParseNewHmi(uint8_t *hmi_data, uint32_t hmi_size) {
     }
 
     if ((hmi_mdi->reverb = _WM_init_reverb(_WM_SampleRate, _WM_reverb_room_width, _WM_reverb_room_length, _WM_reverb_listen_posx, _WM_reverb_listen_posy)) == NULL) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to init reverb", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, 0);
         goto _hmi_end;
     }
 
