@@ -69,6 +69,11 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
     float sample_count_f = 0;
     float sample_remainder = 0;
 
+    if (hmp_size < 776) {
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "file too short", 0);
+        return NULL;
+    }
+
     if (memcmp(hmp_data, "HMIMIDIP", 8)) {
         _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_NOT_HMP, NULL, 0);
         return NULL;
@@ -77,6 +82,10 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
     hmp_size -= 8;
 
     if (!memcmp(hmp_data, "013195", 6)) {
+        if (hmp_size < 896) {
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "file too short", 0);
+            return NULL;
+        }
         hmp_data += 6;
         hmp_size -= 6;
         is_hmp2 = 1;
@@ -103,7 +112,7 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
     hmp_file_length += (*hmp_data++ << 24);
     hmp_size -= 4;
 
-    UNUSED(hmp_file_length);
+    WMIDI_UNUSED(hmp_file_length);
 
     // Next 12 bytes are normally \0 so skipping over them
     hmp_data += 12;
@@ -115,6 +124,11 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
     hmp_chunks += (*hmp_data++ << 24);
     hmp_size -= 4;
 
+    if (!hmp_chunks) {
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(no tracks)", 0);
+        return NULL;
+    }
+
     // Still decyphering what this is
     hmp_unknown = *hmp_data++;
     hmp_unknown += (*hmp_data++ << 8);
@@ -122,7 +136,7 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
     hmp_unknown += (*hmp_data++ << 24);
     hmp_size -= 4;
 
-    UNUSED(hmp_unknown);
+    WMIDI_UNUSED(hmp_unknown);
 
     // Defaulting: experimenting has found this to be the ideal value
     hmp_divisions = 60;
@@ -134,6 +148,11 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
     hmp_bpm += (*hmp_data++ << 24);
     hmp_size -= 4;
 
+    if (!hmp_bpm) {
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, "(bad bpm)", 0);
+        return NULL;
+    }
+
     /* Slow but needed for accuracy */
     if ((_WM_MixerOptions & WM_MO_ROUNDTEMPO)) {
         tempo_f = (float) (60000000 / hmp_bpm) + 0.5f;
@@ -156,7 +175,7 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
     // DEBUG
     //fprintf(stderr,"DEBUG: ??DIVISIONS??: %u, BPM: %u, ??SONG TIME??: %u:%.2u\r\n",hmp_divisions, hmp_bpm, (hmp_song_time / 60), (hmp_song_time % 60));
 
-    UNUSED(hmp_song_time);
+    WMIDI_UNUSED(hmp_song_time);
 
     if (is_hmp2) {
         hmp_data += 840;
@@ -180,6 +199,7 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
     smallest_delta = 0xffffffff;
     // store chunk info for use, and check chunk lengths
     for (i = 0; i < hmp_chunks; i++) {
+        /* FIXME: add proper size checks!!! */
         hmp_chunk[i] = hmp_data;
         chunk_ofs[i] = 0;
 
@@ -189,7 +209,7 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
         chunk_num += (*hmp_data++ << 24);
         chunk_ofs[i] += 4;
 
-        UNUSED(chunk_num);
+        WMIDI_UNUSED(chunk_num);
 
         chunk_length[i] = *hmp_data++;
         chunk_length[i] += (*hmp_data++ << 8);
@@ -209,7 +229,7 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
         hmp_track += (*hmp_data++ << 24);
         chunk_ofs[i] += 4;
 
-        UNUSED(hmp_track);
+        WMIDI_UNUSED(hmp_track);
 
         // Start of Midi Data
         chunk_delta[i] = 0;
@@ -291,7 +311,7 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
                             tempo_f = 500000.0f;
 
                         // DEBUG
-                        fprintf(stderr,"DEBUG: Tempo change %f\r\n", tempo_f);
+                        // fprintf(stderr,"DEBUG: Tempo change %f\r\n", tempo_f);
                     }
                     hmp_chunk[i] += setup_ret;
                     chunk_length[i] -= setup_ret;
@@ -336,7 +356,7 @@ _WM_ParseNewHmp(uint8_t *hmp_data, uint32_t hmp_size) {
     }
 
     if ((hmp_mdi->reverb = _WM_init_reverb(_WM_SampleRate, _WM_reverb_room_width, _WM_reverb_room_length, _WM_reverb_listen_posx, _WM_reverb_listen_posy)) == NULL) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to init reverb", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, 0);
         goto _hmp_end;
     }
 
