@@ -179,11 +179,26 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
             _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(bad track size)", 0);
             goto _end;
         }
-        if ((midi_data[tmp_val - 3] != 0xFF)
-                || (midi_data[tmp_val - 2] != 0x2F)
-                || (midi_data[tmp_val - 1] != 0x00)) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(missing EOT)", 0);
-            goto _end;
+        if ((midi_data[tmp_val - 3] != 0xFF) ||
+            (midi_data[tmp_val - 2] != 0x2F) ||
+            (midi_data[tmp_val - 1] != 0x00)) {
+            /*
+             * Lets do some additional checks
+             * We can ignore this for type 0
+             */
+            if (midi_type != 0) {
+                /*
+                 * Lets also ignore for editors that have added
+                 * an extra byte to the end of the track
+                 */
+                if ((tmp_val < 4)                    ||
+                    (midi_data[tmp_val - 4] != 0xFF) ||
+                    (midi_data[tmp_val - 3] != 0x2F) ||
+                    (midi_data[tmp_val - 2] != 0x00)) {
+                    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(missing EOT)", 0);
+                    goto _end;
+                }
+            }
         }
         tracks[i] = midi_data;
         track_size[i] = tmp_val;
@@ -236,8 +251,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
                 if (track_delta[i]) {
                     track_delta[i] -= subtract_delta;
                     if (track_delta[i]) {
-                        if ((!smallest_delta)
-                             || (smallest_delta > track_delta[i])) {
+                        if (!smallest_delta || (smallest_delta > track_delta[i])) {
                             smallest_delta = track_delta[i];
                         }
                         continue;
@@ -351,8 +365,13 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
                     } while (*tracks[i] > 0x7f);
                 }
                 if (!track_size[i]) {
-                    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
-                    goto _end;
+                    if (midi_type != 0) {
+                        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
+                        goto _end;
+                    } else {
+                        track_end[i] = 1;
+                        goto NEXT_TRACK2;
+                    }
                 }
                 track_delta[i] = (track_delta[i] << 7) + (*tracks[i] & 0x7F);
                 tracks[i]++;
@@ -366,7 +385,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
                 mdi->extra_info.approx_total_samples += sample_count;
             NEXT_TRACK2:
                 smallest_delta = track_delta[i]; /* Added just to keep Xcode happy */
-                UNUSED(smallest_delta); /* Added to just keep clang happy */
+                WMIDI_UNUSED(smallest_delta); /* Added to just keep clang happy */
             } while (track_end[i] == 0);
         }
     }
@@ -374,7 +393,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
     if ((mdi->reverb = _WM_init_reverb(_WM_SampleRate, _WM_reverb_room_width,
             _WM_reverb_room_length, _WM_reverb_listen_posx, _WM_reverb_listen_posy))
           == NULL) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to init reverb", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, 0);
         goto _end;
     }
 
@@ -420,7 +439,7 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
     uint32_t divisions = 96;
     uint32_t tempo = 500000;
     float samples_per_tick = 0;
-    uint32_t value = 0;
+    unsigned long int value = 0;
     float value_f = 0;
     struct _event *event = mdi->events;
     uint32_t track_size = 0;
@@ -472,14 +491,16 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
 
     do {
         /* TODO Is there a better way? */
-        if (event->do_event == _WM_do_midi_divisions) {
+        switch (event->evtype) {
+        case ev_midi_divisions:
             // DEBUG
             // fprintf(stderr,"Division: %u\r\n",event->event_data.data);
             divisions = event->event_data.data.value;
             (*out)[12] = (divisions >> 8) & 0xff;
             (*out)[13] = divisions & 0xff;
             samples_per_tick = _WM_GetSamplesPerTick(divisions, tempo);
-        } else if (event->do_event == _WM_do_note_off) {
+            break;
+        case ev_note_off:
             // DEBUG
             // fprintf(stderr,"Note Off: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0x80 | event->event_data.channel)) {
@@ -488,7 +509,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = (event->event_data.data.value >> 8) & 0xff;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_note_on) {
+            break;
+        case ev_note_on:
             // DEBUG
             // fprintf(stderr,"Note On: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0x90 | event->event_data.channel)) {
@@ -497,7 +519,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = (event->event_data.data.value >> 8) & 0xff;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_aftertouch) {
+            break;
+        case ev_aftertouch:
             // DEBUG
             // fprintf(stderr,"Aftertouch: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xa0 | event->event_data.channel)) {
@@ -506,7 +529,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = (event->event_data.data.value >> 8) & 0xff;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_bank_select) {
+            break;
+        case ev_control_bank_select:
             // DEBUG
             // fprintf(stderr,"Control Bank Select: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -515,7 +539,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 0;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_data_entry_course) {
+            break;
+        case ev_control_data_entry_course:
             // DEBUG
             // fprintf(stderr,"Control Data Entry Course: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -524,7 +549,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 6;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_channel_volume) {
+            break;
+        case ev_control_channel_volume:
             // DEBUG
             // fprintf(stderr,"Control Channel Volume: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -533,7 +559,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 7;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_channel_balance) {
+            break;
+        case ev_control_channel_balance:
             // DEBUG
             // fprintf(stderr,"Control Channel Balance: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -542,7 +569,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 8;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_channel_pan) {
+            break;
+        case ev_control_channel_pan:
             // DEBUG
             // fprintf(stderr,"Control Channel Pan: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -551,7 +579,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 10;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_channel_expression) {
+            break;
+        case ev_control_channel_expression:
             // DEBUG
             // fprintf(stderr,"Control Channel Expression: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -560,7 +589,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 11;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_data_entry_fine) {
+            break;
+        case ev_control_data_entry_fine:
             // DEBUG
             // fprintf(stderr,"Control Data Entry Fine: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -569,7 +599,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 38;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_channel_hold) {
+            break;
+        case ev_control_channel_hold:
             // DEBUG
             // fprintf(stderr,"Control Channel Hold: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -578,7 +609,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 64;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_data_increment) {
+            break;
+        case ev_control_data_increment:
             // DEBUG
             // fprintf(stderr,"Control Data Increment: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -587,7 +619,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 96;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_data_decrement) {
+            break;
+        case ev_control_data_decrement:
             // DEBUG
             //fprintf(stderr,"Control Data Decrement: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -596,7 +629,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 97;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_non_registered_param_fine) {
+            break;
+        case ev_control_non_registered_param_fine:
             // DEBUG
             // fprintf(stderr,"Control Non Registered Param: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -605,7 +639,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 98;
             (*out)[out_ofs++] = event->event_data.data.value & 0x7f;
-        } else if (event->do_event == _WM_do_control_non_registered_param_course) {
+            break;
+        case ev_control_non_registered_param_course:
             // DEBUG
             // fprintf(stderr,"Control Non Registered Param: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -614,7 +649,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 99;
             (*out)[out_ofs++] = (event->event_data.data.value >> 7) & 0x7f;
-        } else if (event->do_event == _WM_do_control_registered_param_fine) {
+            break;
+        case ev_control_registered_param_fine:
             // DEBUG
             // fprintf(stderr,"Control Registered Param Fine: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -623,7 +659,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 100;
             (*out)[out_ofs++] = event->event_data.data.value & 0x7f;
-        } else if (event->do_event == _WM_do_control_registered_param_course) {
+            break;
+        case ev_control_registered_param_course:
             // DEBUG
             // fprintf(stderr,"Control Registered Param Course: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -632,7 +669,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 101;
             (*out)[out_ofs++] = (event->event_data.data.value >> 7) & 0x7f;
-        } else if (event->do_event == _WM_do_control_channel_sound_off) {
+            break;
+        case ev_control_channel_sound_off:
             // DEBUG
             // fprintf(stderr,"Control Channel Sound Off: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -641,7 +679,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 120;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_channel_controllers_off) {
+            break;
+        case ev_control_channel_controllers_off:
             // DEBUG
             // fprintf(stderr,"Control Channel Controllers Off: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -650,7 +689,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 121;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_channel_notes_off) {
+            break;
+        case ev_control_channel_notes_off:
             // DEBUG
             // fprintf(stderr,"Control Channel Notes Off: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -659,7 +699,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = 123;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_control_dummy) {
+            break;
+        case ev_control_dummy:
             // DEBUG
             // fprintf(stderr,"Control Dummy Event: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xb0 | event->event_data.channel)) {
@@ -668,7 +709,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = (event->event_data.data.value >> 8) & 0xff;
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_patch) {
+            break;
+        case ev_patch:
             // DEBUG
             // fprintf(stderr,"Patch: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xc0 | event->event_data.channel)) {
@@ -676,7 +718,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
                 running_event = (*out)[out_ofs - 1];
             }
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_channel_pressure) {
+            break;
+        case ev_channel_pressure:
             // DEBUG
             // fprintf(stderr,"Channel Pressure: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xd0 | event->event_data.channel)) {
@@ -684,7 +727,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
                 running_event = (*out)[out_ofs - 1];
             }
             (*out)[out_ofs++] = event->event_data.data.value & 0xff;
-        } else if (event->do_event == _WM_do_pitch) {
+            break;
+        case ev_pitch:
             // DEBUG
             // fprintf(stderr,"Pitch: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             if (running_event != (0xe0 | event->event_data.channel)) {
@@ -693,7 +737,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             }
             (*out)[out_ofs++] = event->event_data.data.value & 0x7f;
             (*out)[out_ofs++] = (event->event_data.data.value >> 7) & 0x7f;
-        } else if (event->do_event == _WM_do_sysex_roland_drum_track) {
+            break;
+        case ev_sysex_roland_drum_track: {
             // DEBUG
             // fprintf(stderr,"Sysex Roland Drum Track: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
             uint8_t foo[] = {0xf0, 0x09, 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x15, 0x00, 0xf7};
@@ -708,28 +753,32 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             memcpy(&((*out)[out_ofs]),foo,11);
             out_ofs += 11;
             running_event = 0;
-        } else if (event->do_event == _WM_do_sysex_gm_reset) {
+          } break;
+        case ev_sysex_gm_reset: {
             // DEBUG
             // fprintf(stderr,"Sysex GM Reset\r\n");
             uint8_t foo[] = {0xf0, 0x05, 0x7e, 0x7f, 0x09, 0x01, 0xf7};
             memcpy(&((*out)[out_ofs]),foo,7);
             out_ofs += 7;
             running_event = 0;
-        } else if (event->do_event == _WM_do_sysex_roland_reset) {
+          } break;
+        case ev_sysex_roland_reset: {
             // DEBUG
             // fprintf(stderr,"Sysex Roland Reset\r\n");
             uint8_t foo[] = {0xf0, 0x0a, 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x7f, 0x00, 0x41, 0xf7};
             memcpy(&((*out)[out_ofs]),foo,12);
             out_ofs += 12;
             running_event = 0;
-        } else if (event->do_event == _WM_do_sysex_yamaha_reset) {
+          } break;
+        case ev_sysex_yamaha_reset: {
             // DEBUG
             // fprintf(stderr,"Sysex Yamaha Reset\r\n");
             uint8_t foo[] = {0xf0, 0x08, 0x43, 0x10, 0x4c, 0x00, 0x00, 0x7e, 0x00, 0xf7};
             memcpy(&((*out)[out_ofs]),foo,10);
             out_ofs += 10;
             running_event = 0;
-        } else if (event->do_event == _WM_do_meta_endoftrack) {
+          } break;
+        case ev_meta_endoftrack:
             // DEBUG
             // fprintf(stderr,"End Of Track\r\n");
             if ((!(_WM_MixerOptions & WM_MO_SAVEASTYPE0)) && (mdi->is_type2)) {
@@ -743,7 +792,7 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
                 (*out)[track_start - 2] = (track_size >> 8) & 0xff;
                 (*out)[track_start - 1] = track_size & 0xff;
 
-                if (event[1].do_event != NULL) {
+                if (event[1].evtype != ev_null) {
                     (*out)[out_ofs++] = 'M';
                     (*out)[out_ofs++] = 'T';
                     (*out)[out_ofs++] = 'r';
@@ -759,7 +808,7 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
                 }
             }
             goto NEXT_EVENT;
-        } else if (event->do_event == _WM_do_meta_tempo) {
+        case ev_meta_tempo:
             // DEBUG
             // fprintf(stderr,"Tempo: %u\r\n",event->event_data.data);
             tempo = event->event_data.data.value & 0xffffff;
@@ -775,7 +824,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             (*out)[out_ofs++] = (tempo & 0xff0000) >> 16;
             (*out)[out_ofs++] = (tempo & 0xff00) >> 8;
             (*out)[out_ofs++] = (tempo & 0xff);
-        } else if (event->do_event == _WM_do_meta_timesignature) {
+            break;
+        case ev_meta_timesignature:
             // DEBUG
             // fprintf(stderr,"Time Signature: %x\r\n",event->event_data.data);
             (*out)[out_ofs++] = 0xff;
@@ -785,7 +835,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff0000) >> 16;
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff00) >> 8;
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff);
-        } else if (event->do_event == _WM_do_meta_keysignature) {
+            break;
+        case ev_meta_keysignature:
             // DEBUG
             // fprintf(stderr,"Key Signature: %x\r\n",event->event_data.data);
             (*out)[out_ofs++] = 0xff;
@@ -793,7 +844,8 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             (*out)[out_ofs++] = 0x02;
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff00) >> 8;
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff);
-        } else if (event->do_event == _WM_do_meta_sequenceno) {
+            break;
+        case ev_meta_sequenceno:
             // DEBUG
             // fprintf(stderr,"Sequence Number: %x\r\n",event->event_data.data);
             (*out)[out_ofs++] = 0xff;
@@ -801,21 +853,24 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             (*out)[out_ofs++] = 0x02;
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff00) >> 8;
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff);
-        } else if (event->do_event == _WM_do_meta_channelprefix) {
+            break;
+        case ev_meta_channelprefix:
             // DEBUG
             // fprintf(stderr,"Channel Prefix: %x\r\n",event->event_data.data);
             (*out)[out_ofs++] = 0xff;
             (*out)[out_ofs++] = 0x20;
             (*out)[out_ofs++] = 0x01;
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff);
-        } else if (event->do_event == _WM_do_meta_portprefix) {
+            break;
+        case ev_meta_portprefix:
             // DEBUG
             // fprintf(stderr,"Port Prefix: %x\r\n",event->event_data.data);
             (*out)[out_ofs++] = 0xff;
             (*out)[out_ofs++] = 0x21;
             (*out)[out_ofs++] = 0x01;
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff);
-        } else if (event->do_event == _WM_do_meta_smpteoffset) {
+            break;
+        case ev_meta_smpteoffset:
             // DEBUG
             // fprintf(stderr,"SMPTE Offset: %x\r\n",event->event_data.data);
             (*out)[out_ofs++] = 0xff;
@@ -829,44 +884,45 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff0000) >> 16;
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff00) >> 8;
             (*out)[out_ofs++] = (event->event_data.data.value & 0xff);
+            break;
 
-        } else if (event->do_event == _WM_do_meta_text) {
+        case ev_meta_text:
             (*out)[out_ofs++] = 0xff;
             (*out)[out_ofs++] = 0x01;
 
             goto _WRITE_TEXT;
 
-        } else if (event->do_event == _WM_do_meta_copyright) {
+        case ev_meta_copyright:
             (*out)[out_ofs++] = 0xff;
             (*out)[out_ofs++] = 0x02;
 
             goto _WRITE_TEXT;
 
-        } else if (event->do_event == _WM_do_meta_trackname) {
+        case ev_meta_trackname:
             (*out)[out_ofs++] = 0xff;
             (*out)[out_ofs++] = 0x03;
 
             goto _WRITE_TEXT;
 
-        } else if (event->do_event == _WM_do_meta_instrumentname) {
+        case ev_meta_instrumentname:
             (*out)[out_ofs++] = 0xff;
             (*out)[out_ofs++] = 0x04;
 
             goto _WRITE_TEXT;
 
-        } else if (event->do_event == _WM_do_meta_lyric) {
+        case ev_meta_lyric:
             (*out)[out_ofs++] = 0xff;
             (*out)[out_ofs++] = 0x05;
 
             goto _WRITE_TEXT;
 
-        } else if (event->do_event == _WM_do_meta_marker) {
+        case ev_meta_marker:
             (*out)[out_ofs++] = 0xff;
             (*out)[out_ofs++] = 0x06;
 
             goto _WRITE_TEXT;
 
-        } else if (event->do_event == _WM_do_meta_cuepoint) {
+        case ev_meta_cuepoint:
             (*out)[out_ofs++] = 0xff;
             (*out)[out_ofs++] = 0x07;
 
@@ -884,10 +940,11 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
 
             memcpy(&(*out)[out_ofs], event->event_data.data.string, value);
             out_ofs += value;
+            break;
 
-        } else {
+        default:
             // DEBUG
-            fprintf(stderr,"Unknown Event %.2x %.4x\n",event->event_data.channel, event->event_data.data.value);
+            // fprintf(stderr,"Unknown Event %.2x %.4x\n",event->event_data.channel, event->event_data.data.value);
             event++;
             continue;
         }
@@ -909,7 +966,7 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
         (*out)[out_ofs++] = (value & 0x7f);
     NEXT_EVENT:
         event++;
-    } while (event->do_event != NULL);
+    } while (event->evtype != ev_null);
 
     if ((_WM_MixerOptions & WM_MO_SAVEASTYPE0) || (!mdi->is_type2)) {
         /* Write end of track marker */
