@@ -1,5 +1,5 @@
 /*
- * mus_wm.c -- Midi Wavetable Processing library
+ * mus.c -- Midi Wavetable Processing library
  *
  * Copyright (C) WildMIDI Developers 2001-2016
  *
@@ -71,8 +71,8 @@ _WM_ParseNewMus(uint8_t *mus_data, uint32_t mus_size) {
     float sample_remainder = 0;
     uint16_t pitchbend_tmp = 0;
 
-    if (mus_size < 17) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_NOT_MUS, "File too short", 0);
+    if (mus_size < 18) {
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "file too short", 0);
         return NULL;
     }
 
@@ -90,8 +90,8 @@ _WM_ParseNewMus(uint8_t *mus_data, uint32_t mus_size) {
     mus_ch_cnt1 = (mus_data[9] << 8) | mus_data[8];
     mus_ch_cnt2 = (mus_data[11] << 8) | mus_data[10];
 
-    UNUSED(mus_ch_cnt1);
-    UNUSED(mus_ch_cnt2);
+    WMIDI_UNUSED(mus_ch_cnt1);
+    WMIDI_UNUSED(mus_ch_cnt2);
 
     // Number of instruments defined
     mus_no_instr = (mus_data[13] << 8) | mus_data[12];
@@ -101,7 +101,7 @@ _WM_ParseNewMus(uint8_t *mus_data, uint32_t mus_size) {
 
     // Check that we have enough data to check the rest
     if (mus_size < (mus_data_ofs + (mus_no_instr << 1) + mus_song_len)) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_NOT_MUS, "File too short", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "file too short", 0);
         return NULL;
     }
 
@@ -134,7 +134,7 @@ _WM_ParseNewMus(uint8_t *mus_data, uint32_t mus_size) {
 
     // lets do this
     do {
-        // Build the event
+     // Build the event
     _mus_build_event:
 #if 1
         // Mus drums happen on channel 15, swap channel 9 & 15
@@ -341,7 +341,7 @@ _WM_ParseNewMus(uint8_t *mus_data, uint32_t mus_size) {
 _mus_end_of_song:
     // Finalise mdi structure
     if ((mus_mdi->reverb = _WM_init_reverb(_WM_SampleRate, _WM_reverb_room_width, _WM_reverb_room_length, _WM_reverb_listen_posx, _WM_reverb_listen_posy)) == NULL) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to init reverb", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, 0);
         goto _mus_end;
     }
     _WM_midi_setup_endoftrack(mus_mdi);
