@@ -71,6 +71,8 @@
 
 #include "wm_error.h"
 #include "file_io.h"
+void* (*_WM_BufferFile)(const char *, uint32_t *) = _WM_BufferFileImpl;
+void  (*_WM_FreeBufferFile)(void*)                = _WM_FreeBufferFileImpl;
 
 #ifdef WILDMIDI_AMIGA
 static long AMIGA_filesize (const char *path) {
@@ -109,7 +111,7 @@ static void AMIGA_close (BPTR fd) {
 }
 #endif
 
-void *_WM_BufferFile(const char *filename, uint32_t *size) {
+void *_WM_BufferFileImpl(const char *filename, uint32_t *size) {
     char *buffer_file = NULL;
     uint8_t *data;
 #ifdef __DJGPP__
@@ -127,7 +129,7 @@ void *_WM_BufferFile(const char *filename, uint32_t *size) {
 #elif defined(WILDMIDI_AMIGA)
     BPTR buffer_fd;
     long filsize;
-#elif defined(_3DS) || defined(GEKKO)
+#elif defined(_3DS) || defined(GEKKO) || defined(__vita__) || defined(__SWITCH__) || defined(__riscos__)
     int buffer_fd;
     struct stat buffer_stat;
 #else /* unix builds */
@@ -149,7 +151,7 @@ void *_WM_BufferFile(const char *filename, uint32_t *size) {
         if (home) {
             buffer_file = (char *) malloc(strlen(filename) + strlen(home) + 1);
             if (buffer_file == NULL) {
-                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, filename, errno);
+                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
                 return NULL;
             }
             strcpy(buffer_file, home);
@@ -160,7 +162,7 @@ void *_WM_BufferFile(const char *filename, uint32_t *size) {
         if (cwdresult != NULL)
             buffer_file = (char *) malloc(strlen(filename) + strlen(buffer_dir) + 2);
         if (buffer_file == NULL || cwdresult == NULL) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, filename, errno);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
             return NULL;
         }
         strcpy(buffer_file, buffer_dir);
@@ -173,7 +175,7 @@ void *_WM_BufferFile(const char *filename, uint32_t *size) {
     if (buffer_file == NULL) {
         buffer_file = (char *) malloc(strlen(filename) + 1);
         if (buffer_file == NULL) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, filename, errno);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
             return NULL;
         }
         strcpy(buffer_file, filename);
@@ -233,7 +235,7 @@ void *_WM_BufferFile(const char *filename, uint32_t *size) {
     /* +1 needed for parsing text files without a newline at the end */
     data = (uint8_t *) malloc(*size + 1);
     if (data == NULL) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, filename, errno);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
         free(buffer_file);
         return NULL;
     }
@@ -275,3 +277,7 @@ void *_WM_BufferFile(const char *filename, uint32_t *size) {
     data[*size] = '\0';
     return data;
 }
+
+void _WM_FreeBufferFileImpl(void *buf) {
+    free(buf);
+}
