@@ -73,7 +73,7 @@ static int convert_8s(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, 0, "calloc failed", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
 
     return -1;
 }
@@ -126,7 +126,7 @@ static int convert_8sp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -152,7 +152,7 @@ static int convert_8sr(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_REVERSE;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -203,7 +203,7 @@ static int convert_8srp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -223,7 +223,7 @@ static int convert_8u(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_UNSIGNED;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -275,7 +275,7 @@ static int convert_8up(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -301,7 +301,7 @@ static int convert_8ur(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_REVERSE | SAMPLE_UNSIGNED;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -351,7 +351,7 @@ static int convert_8urp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -375,7 +375,7 @@ static int convert_16s(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->data_length >>= 1;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -435,7 +435,7 @@ static int convert_16sp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -465,7 +465,7 @@ static int convert_16sr(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_REVERSE;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -520,7 +520,7 @@ static int convert_16srp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -544,7 +544,7 @@ static int convert_16u(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_UNSIGNED;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -604,7 +604,7 @@ static int convert_16up(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -634,7 +634,7 @@ static int convert_16ur(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_REVERSE | SAMPLE_UNSIGNED;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -689,7 +689,7 @@ static int convert_16urp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
     return -1;
 }
 
@@ -700,6 +700,7 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
     uint32_t gus_size;
     uint32_t gus_ptr;
     uint8_t no_of_samples;
+    uint8_t envsusreltime, envreltime;
     struct _sample *gus_sample = NULL;
     struct _sample *first_gus_sample = NULL;
     uint32_t i = 0;
@@ -724,7 +725,7 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
     };
     uint32_t tmp_loop;
 
-    UNUSED(fix_release);
+    WMIDI_UNUSED(fix_release);
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__); SAMPLE_CONVERT_DEBUG(filename);
 
@@ -733,23 +734,23 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
     }
     if (gus_size < 239) {
         _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, filename, 0);
-        free(gus_patch);
+        _WM_FreeBufferFile(gus_patch);
         return NULL;
     }
     if (memcmp(gus_patch, "GF1PATCH110\0ID#000002", 22)
             && memcmp(gus_patch, "GF1PATCH100\0ID#000002", 22)) {
         _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, filename, 0);
-        free(gus_patch);
+        _WM_FreeBufferFile(gus_patch);
         return NULL;
     }
     if (gus_patch[82] > 1) {
         _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, filename, 0);
-        free(gus_patch);
+        _WM_FreeBufferFile(gus_patch);
         return NULL;
     }
     if (gus_patch[151] > 1) {
         _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, filename, 0);
-        free(gus_patch);
+        _WM_FreeBufferFile(gus_patch);
         return NULL;
     }
 
@@ -768,8 +769,8 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
             gus_sample = gus_sample->next;
         }
         if (gus_sample == NULL) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, filename, 0);
-            free(gus_patch);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, 0);
+            _WM_FreeBufferFile(gus_patch);
             return NULL;
         }
 
@@ -825,11 +826,22 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
                             | ((gus_sample->loop_fraction & 0xf0) >> 4);
         }
 
-        // kill "echo" envelopes because users dont want/like them
-        // and this is what timidity does which is why we sounded different
-        // NOTE: This may cause some pats to sound different to what their authers intended
-        gus_patch[gus_ptr + 41] = 0x3f;
-        gus_patch[gus_ptr + 42] = 0x3f;
+        // All sorts of annoying things happen with pat files.
+        // One of them is that the sustained release time and
+        // normal release time gets mixed up because software got muddled
+        envsusreltime = env_time_table[gus_patch[gus_ptr + 40]];
+        envreltime = env_time_table[gus_patch[gus_ptr + 41]];
+        if (envsusreltime < envreltime) {
+            // EXPERIMENTAL
+            gus_patch[gus_ptr + 40] = gus_patch[gus_ptr + 41];
+            // timidity does this:
+            gus_patch[gus_ptr + 41] = 0x3f;
+            gus_patch[gus_ptr + 42] = 0x3f;
+
+            gus_patch[gus_ptr + 46] = gus_patch[gus_ptr + 47];
+            gus_patch[gus_ptr + 47] = 0;
+            gus_patch[gus_ptr + 48] = 0;
+        }
 
         // lets set up the envelope data
         for (i = 0; i < 6; i++) {
@@ -843,7 +855,7 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
                 GUSPAT_INT_DEBUG("Envelope Rate",gus_sample->env_rate[i]); GUSPAT_INT_DEBUG("GUSPAT Rate",env_rate);
                 if (gus_sample->env_rate[i] == 0) {
                     _WM_DEBUG_MSG("%s: Warning: found invalid envelope(%u) rate setting in %s. Using %f instead.",
-                            __FUNCTION__, i, filename, env_time_table[63]);
+                                  __FUNCTION__, i, filename, env_time_table[63]);
                     gus_sample->env_rate[i] = (int32_t) (4194303.0f
                             / ((float) _WM_SampleRate * env_time_table[63]));
                     GUSPAT_FLOAT_DEBUG("Envelope Time",env_time_table[63]);
@@ -866,7 +878,7 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
         if (do_convert[(((gus_sample->modes & 0x18) >> 1)
                 | (gus_sample->modes & 0x03))](&gus_patch[gus_ptr], gus_sample)
                 == -1) {
-            free(gus_patch);
+            _WM_FreeBufferFile(gus_patch);
             return NULL;
         }
 
@@ -905,6 +917,6 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
         gus_sample->data_length = gus_sample->data_length << 10;
         no_of_samples--;
     }
-    free(gus_patch);
+    _WM_FreeBufferFile(gus_patch);
     return first_gus_sample;
 }
