@@ -23,16 +23,16 @@
 
 #include "config.h"
 
-#include <sys/types.h>
 #include <stdint.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/stat.h>
 
 #if defined(__DJGPP__)
+#include <sys/types.h>
+#include <fcntl.h>
+#include <sys/stat.h>
 #include "getopt_long.h"
 #include <conio.h>
 #define getopt dj_getopt /* hack */
@@ -44,29 +44,73 @@
 #ifdef AUDIODRV_DOSSB
 #include "dossb.h"
 #endif
-#endif
 
-#if (defined _WIN32) || (defined __CYGWIN__)
+#elif (defined _WIN32) || (defined __CYGWIN__)
+#include <sys/types.h>
+#include <fcntl.h>
+#include <sys/stat.h>
 #include <conio.h>
 #include <windows.h>
 #include <mmsystem.h>
 #define msleep(s) Sleep((s))
 #include <io.h>
-#undef close
-#define close _close
-#undef open
-#define open _open
-#undef read
-#define read _read
-#undef write
-#define write _write
-#undef lseek
-#define lseek _lseek
 #include "getopt_long.h"
+#ifdef __WATCOMC__
+#define _putch putch
 #endif
 
-#if !defined(_WIN32) && !defined(__DJGPP__) /* unix build */
+#elif defined(__OS2__) || defined(__EMX__)
+#define INCL_DOS
+#define INCL_DOSERRORS
+#define INCL_OS2MM
+#ifdef __EMX__
+#define INCL_KBD
+#define INCL_VIO
+#endif
+#include <os2.h>
+#include <os2me.h>
+#include <conio.h>
+#define msleep(s) DosSleep((s))
+#include <fcntl.h>
+#include <io.h>
+#include "getopt_long.h"
+#ifdef __EMX__
+#include <sys/types.h> /* for off_t typedef */
+int putch (int c) {
+    char ch = c;
+    VioWrtTTY(&ch, 1, 0);
+    return c;
+}
+int kbhit (void) {
+    KBDKEYINFO k;
+    if (KbdPeek(&k, 0))
+        return 0;
+    return (k.fbStatus & KBDTRF_FINAL_CHAR_IN);
+}
+#endif
+
+#elif defined(WILDMIDI_AMIGA)
+extern void amiga_sysinit (void);
+extern int amiga_usleep(unsigned long millisec);
+#define msleep(s) amiga_usleep((s)*1000)
+extern int amiga_getch (unsigned char *ch);
+#include <proto/exec.h>
+#include <proto/dos.h>
+#include "getopt_long.h"
+#ifdef AUDIODRV_AHI
+#include <devices/ahi.h>
+#ifdef __amigaos4__
+#define SHAREDMEMFLAG MEMF_SHARED
+#else
+#define SHAREDMEMFLAG MEMF_PUBLIC
+#endif
+#endif
+
+#else /* unix build */
 static int msleep(unsigned long millisec);
+#include <sys/types.h>
+#include <fcntl.h>
+#include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
 #include <getopt.h>
@@ -82,8 +126,13 @@ static int msleep(unsigned long millisec);
 #   include <soundcard.h> /* less common, but exists. */
 #   endif
 #elif defined AUDIODRV_OPENAL
+#   ifndef __APPLE__
 #   include <al.h>
 #   include <alc.h>
+#   else
+#   include <OpenAL/al.h>
+#   include <OpenAL/alc.h>
+#   endif
 #endif
 #endif /* !_WIN32, !__DJGPP__ (unix build) */
 
@@ -209,7 +258,128 @@ static int (*send_output)(int8_t *output_data, int output_size);
 static void (*close_output)(void);
 static void (*pause_output)(void);
 static void (*resume_output)(void);
+
+#define wmidi_geterrno() errno /* generic case */
+#if defined(_WIN32)
 static int audio_fd = -1;
+#define WM_IS_BADF(_fd) ((_fd)<0)
+#define WM_BADF -1
+static inline int wmidi_fileexists (const char *path) {
+    return (GetFileAttributes(path) != INVALID_FILE_ATTRIBUTES);
+}
+static inline int wmidi_open_write (const char *path) {
+    return _open(path, (O_RDWR | O_CREAT | O_TRUNC | O_BINARY), 0664);
+}
+static inline void wmidi_close (int fd) {
+    _close(fd);
+}
+static inline long wmidi_seekset (int fd, long ofs) {
+    return _lseek(fd, ofs, SEEK_SET);
+}
+static inline int wmidi_write (int fd, const void *buf, size_t size) {
+    return _write(fd, buf, size);
+}
+
+#elif defined(__DJGPP__)
+static int audio_fd = -1;
+#define WM_IS_BADF(_fd) ((_fd)<0)
+#define WM_BADF -1
+static inline int wmidi_fileexists (const char *path) {
+    struct ffblk f;
+    return (findfirst(path, &f, FA_ARCH | FA_RDONLY) == 0);
+}
+static inline int wmidi_open_write (const char *path) {
+    return open(path, (O_RDWR | O_CREAT | O_TRUNC | O_BINARY), 0664);
+}
+static inline void wmidi_close (int fd) {
+    close(fd);
+}
+static inline off_t wmidi_seekset (int fd, off_t ofs) {
+    return lseek(fd, ofs, SEEK_SET);
+}
+static inline int wmidi_write (int fd, const void *buf, size_t size) {
+    return write(fd, buf, size);
+}
+
+#elif defined(__OS2__) || defined(__EMX__)
+static int audio_fd = -1;
+#define WM_IS_BADF(_fd) ((_fd)<0)
+#define WM_BADF -1
+static inline int wmidi_fileexists (const char *path) {
+    int f = open(path, (O_RDONLY | O_BINARY));
+    if (f != -1) { close(f); return 1; } else return 0;
+}
+static inline int wmidi_open_write (const char *path) {
+    return open(path, (O_RDWR | O_CREAT | O_TRUNC | O_BINARY), 0664);
+}
+static inline void wmidi_close (int fd) {
+    close(fd);
+}
+static inline off_t wmidi_seekset (int fd, off_t ofs) {
+    return lseek(fd, ofs, SEEK_SET);
+}
+static inline int wmidi_write (int fd, const void *buf, size_t size) {
+    return write(fd, buf, size);
+}
+
+#elif defined(WILDMIDI_AMIGA)
+static BPTR audio_fd = 0;
+#define WM_IS_BADF(_fd) ((_fd)==0)
+#define WM_BADF 0
+#undef wmidi_geterrno
+static int wmidi_geterrno (void) {
+    switch (IoErr()) {
+    case ERROR_OBJECT_NOT_FOUND: return ENOENT;
+    case ERROR_DISK_FULL: return ENOSPC;
+    }
+    return EIO; /* better ?? */
+}
+static inline int wmidi_fileexists (const char *path) {
+    BPTR fd = Open((const STRPTR)path, MODE_OLDFILE);
+    if (!fd) return 0;
+    Close(fd); return 1;
+}
+static inline BPTR wmidi_open_write (const char *path) {
+    return Open((const STRPTR) path, MODE_NEWFILE);
+}
+static inline LONG wmidi_close (BPTR fd) {
+    return Close(fd);
+}
+static inline LONG wmidi_seekset (BPTR fd, LONG ofs) {
+    return Seek(fd, ofs, OFFSET_BEGINNING);
+}
+static LONG wmidi_write (BPTR fd, /*const*/ void *buf, LONG size) {
+    LONG written = 0, result;
+    unsigned char *p = (unsigned char *)buf;
+    while (written < size) {
+        result = Write(fd, p + written, size - written);
+        if (result < 0) return result;
+        written += result;
+    }
+    return written;
+}
+
+#else /* common posix case */
+static int audio_fd = -1;
+#define WM_IS_BADF(_fd) ((_fd)<0)
+#define WM_BADF -1
+static inline int wmidi_fileexists (const char *path) {
+    struct stat st;
+    return (stat(path, &st) == 0);
+}
+static inline int wmidi_open_write (const char *path) {
+    return open(path, (O_RDWR | O_CREAT | O_TRUNC), (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH));
+}
+static inline int wmidi_close (int fd) {
+    return close(fd);
+}
+static inline off_t wmidi_seekset (int fd, off_t ofs) {
+    return lseek(fd, ofs, SEEK_SET);
+}
+static inline ssize_t wmidi_write (int fd, const void *buf, size_t size) {
+    return write(fd, buf, size);
+}
+#endif
 
 static void pause_output_nop(void) {
 }
@@ -222,47 +392,32 @@ static void resume_output_nop(void) {
 static char midi_file[1024];
 
 static int write_midi_output(void *output_data, int output_size) {
-#ifdef __DJGPP__
-    struct ffblk f;
-#else
-    struct stat st;
-#endif
-
     if (midi_file[0] == '\0')
         return (-1);
 
 /*
  * Test if file already exists 
  */
-#ifdef __DJGPP__
-    if (findfirst(midi_file, &f, FA_ARCH | FA_RDONLY) == 0) {
-#else
-    if (stat(midi_file, &st) == 0) {
-#endif
+    if (wmidi_fileexists(midi_file)) {
         fprintf(stderr, "\rError: %s already exists\r\n", midi_file);
         return (-1);
     }
 
-#if defined(_WIN32) || defined(__DJGPP__)
-    audio_fd = open(midi_file, (O_RDWR | O_CREAT | O_TRUNC | O_BINARY), 0664);
-#else
-    audio_fd = open(midi_file, (O_RDWR | O_CREAT | O_TRUNC),
-                                                            (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH));
-#endif
-    if (audio_fd < 0) {
-        fprintf(stderr, "Error: unable to open file for writing (%s)\r\n", strerror(errno));
+    audio_fd = wmidi_open_write(midi_file);
+    if (WM_IS_BADF(audio_fd)) {
+        fprintf(stderr, "Error: unable to open file for writing (%s)\r\n", strerror(wmidi_geterrno()));
         return (-1);
     }
 
-    if (write(audio_fd, output_data, output_size) < 0) {
-        fprintf(stderr, "\nERROR: failed writing midi (%s)\r\n", strerror(errno));
-        close(audio_fd);
-        audio_fd = -1;
+    if (wmidi_write(audio_fd, output_data, output_size) < 0) {
+        fprintf(stderr, "\nERROR: failed writing midi (%s)\r\n", strerror(wmidi_geterrno()));
+        wmidi_close(audio_fd);
+        audio_fd = WM_BADF;
         return (-1);
     }
 
-    close(audio_fd);
-    audio_fd = -1;
+    wmidi_close(audio_fd);
+    audio_fd = WM_BADF;
     return (0);
 }
 
@@ -296,14 +451,9 @@ static int open_wav_output(void) {
     if (wav_file[0] == '\0')
         return (-1);
 
-#if defined(_WIN32) || defined(__DJGPP__)
-    audio_fd = open(wav_file, (O_RDWR | O_CREAT | O_TRUNC | O_BINARY), 0664);
-#else
-    audio_fd = open(wav_file, (O_RDWR | O_CREAT | O_TRUNC),
-                                                           (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH));
-#endif
-    if (audio_fd < 0) {
-        fprintf(stderr, "Error: unable to open file for writing (%s)\r\n", strerror(errno));
+    audio_fd = wmidi_open_write(wav_file);
+    if (WM_IS_BADF(audio_fd)) {
+        fprintf(stderr, "Error: unable to open file for writing (%s)\r\n", strerror(wmidi_geterrno()));
         return (-1);
     } else {
         uint32_t bytes_per_sec;
@@ -318,10 +468,10 @@ static int open_wav_output(void) {
         wav_hdr[31] = (bytes_per_sec >> 24) & 0xFF;
     }
 
-    if (write(audio_fd, &wav_hdr, 44) < 0) {
-        fprintf(stderr, "ERROR: failed writing wav header (%s)\r\n", strerror(errno));
-        close(audio_fd);
-        audio_fd = -1;
+    if (wmidi_write(audio_fd, wav_hdr, 44) < 0) {
+        fprintf(stderr, "ERROR: failed writing wav header (%s)\r\n", strerror(wmidi_geterrno()));
+        wmidi_close(audio_fd);
+        audio_fd = WM_BADF;
         return (-1);
     }
 
@@ -342,10 +492,10 @@ static int write_wav_output(int8_t *output_data, int output_size) {
         swp[i] = (swp[i] << 8) | (swp[i] >> 8);
     }
 #endif
-    if (write(audio_fd, output_data, output_size) < 0) {
-        fprintf(stderr, "\nERROR: failed writing wav (%s)\r\n", strerror(errno));
-        close(audio_fd);
-        audio_fd = -1;
+    if (wmidi_write(audio_fd, output_data, output_size) < 0) {
+        fprintf(stderr, "\nERROR: failed writing wav (%s)\r\n", strerror(wmidi_geterrno()));
+        wmidi_close(audio_fd);
+        audio_fd = WM_BADF;
         return (-1);
     }
 
@@ -355,7 +505,7 @@ static int write_wav_output(int8_t *output_data, int output_size) {
 
 static void close_wav_output(void) {
     uint8_t wav_count[4];
-    if (audio_fd < 0)
+    if (WM_IS_BADF(audio_fd))
         return;
 
     printf("Finishing and closing wav output\r");
@@ -363,9 +513,9 @@ static void close_wav_output(void) {
     wav_count[1] = (wav_size >> 8) & 0xFF;
     wav_count[2] = (wav_size >> 16) & 0xFF;
     wav_count[3] = (wav_size >> 24) & 0xFF;
-    lseek(audio_fd, 40, SEEK_SET);
-    if (write(audio_fd, &wav_count, 4) < 0) {
-        fprintf(stderr, "\nERROR: failed writing wav (%s)\r\n", strerror(errno));
+    wmidi_seekset(audio_fd, 40);
+    if (wmidi_write(audio_fd, wav_count, 4) < 0) {
+        fprintf(stderr, "\nERROR: failed writing wav (%s)\r\n", strerror(wmidi_geterrno()));
         goto end;
     }
 
@@ -374,16 +524,16 @@ static void close_wav_output(void) {
     wav_count[1] = (wav_size >> 8) & 0xFF;
     wav_count[2] = (wav_size >> 16) & 0xFF;
     wav_count[3] = (wav_size >> 24) & 0xFF;
-    lseek(audio_fd, 4, SEEK_SET);
-    if (write(audio_fd, &wav_count, 4) < 0) {
-        fprintf(stderr, "\nERROR: failed writing wav (%s)\r\n", strerror(errno));
+    wmidi_seekset(audio_fd, 4);
+    if (wmidi_write(audio_fd, wav_count, 4) < 0) {
+        fprintf(stderr, "\nERROR: failed writing wav (%s)\r\n", strerror(wmidi_geterrno()));
         goto end;
     }
 
 end:    printf("\n");
-    if (audio_fd >= 0)
-        close(audio_fd);
-    audio_fd = -1;
+    if (!WM_IS_BADF(audio_fd))
+        wmidi_close(audio_fd);
+    audio_fd = WM_BADF;
 }
 
 #if (defined _WIN32) || (defined __CYGWIN__)
@@ -428,7 +578,7 @@ open_mm_output (void) {
 
     InitializeCriticalSection(&waveCriticalSection);
 
-    if((mm_buffer = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, ((MM_BLOCK_SIZE + sizeof(WAVEHDR)) * MM_BLOCK_COUNT))) == NULL) {
+    if((mm_buffer = (char *) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, ((MM_BLOCK_SIZE + sizeof(WAVEHDR)) * MM_BLOCK_COUNT))) == NULL) {
         fprintf(stderr, "Memory allocation error\r\n");
         return -1;
     }
@@ -524,6 +674,182 @@ close_mm_output (void) {
     mm_blocks = NULL;
 }
 
+#elif (defined(__OS2__) || defined(__EMX__)) && defined(AUDIODRV_OS2DART)
+/* based on Dart code originally written by Kevin Langman for XMP */
+
+#define open_audio_output open_dart_output
+static int write_dart_output (int8_t *output_data, int output_size);
+static void close_dart_output (void);
+
+#define BUFFERCOUNT 4
+
+static MCI_MIX_BUFFER MixBuffers[BUFFERCOUNT];
+static MCI_MIXSETUP_PARMS MixSetupParms;
+static MCI_BUFFER_PARMS BufferParms;
+static MCI_GENERIC_PARMS GenericParms;
+
+static ULONG DeviceID = 0;
+static ULONG bsize = 16;
+static short next = 2;
+static short ready = 1;
+
+static HMTX dart_mutex;
+
+/* Buffer update thread (created and called by DART) */
+static LONG APIENTRY OS2_Dart_UpdateBuffers
+    (ULONG ulStatus, PMCI_MIX_BUFFER pBuffer, ULONG ulFlags) {
+
+    (void) pBuffer;/* unused param */
+
+    if ((ulFlags == MIX_WRITE_COMPLETE) ||
+        ((ulFlags == (MIX_WRITE_COMPLETE | MIX_STREAM_ERROR)) &&
+         (ulStatus == ERROR_DEVICE_UNDERRUN))) {
+        DosRequestMutexSem(dart_mutex, SEM_INDEFINITE_WAIT);
+        ready++;
+        DosReleaseMutexSem(dart_mutex);
+    }
+    return (TRUE);
+}
+
+static int
+open_dart_output(void) {
+    int i;
+    MCI_AMP_OPEN_PARMS AmpOpenParms;
+
+    if (DosCreateMutexSem(NULL, &dart_mutex, 0, 0) != NO_ERROR) {
+        fprintf(stderr, "Failed creating a MutexSem.\r\n");
+        return (-1);
+    }
+
+    /* compute a size for circa 1/4" of playback. */
+    bsize = rate >> 2;
+    bsize <<= 1; /* stereo */
+    bsize <<= 1; /* 16 bit */
+    for (i = 15; i >= 12; i--) {
+        if (bsize & (1 << i))
+            break;
+    }
+    bsize = (1 << i);
+    /* make sure buffer is not greater than 64 Kb: DART can't handle it. */
+    if (bsize > 65536)
+        bsize = 65536;
+
+    MixBuffers[0].pBuffer = NULL; /* marker */
+    memset(&GenericParms, 0, sizeof(MCI_GENERIC_PARMS));
+
+    /* open AMP device */
+    memset(&AmpOpenParms, 0, sizeof(MCI_AMP_OPEN_PARMS));
+    AmpOpenParms.usDeviceID = 0;
+
+    AmpOpenParms.pszDeviceType =
+        (PSZ) MAKEULONG(MCI_DEVTYPE_AUDIO_AMPMIX, 0); /* 0: default waveaudio device */
+
+    if(mciSendCommand(0, MCI_OPEN, MCI_WAIT|MCI_OPEN_TYPE_ID|MCI_OPEN_SHAREABLE,
+                       (PVOID) &AmpOpenParms, 0) != MCIERR_SUCCESS) {
+        fprintf(stderr, "Failed opening DART audio device\r\n");
+        return (-1);
+    }
+
+    DeviceID = AmpOpenParms.usDeviceID;
+
+    /* setup playback parameters */
+    memset(&MixSetupParms, 0, sizeof(MCI_MIXSETUP_PARMS));
+
+    MixSetupParms.ulBitsPerSample = 16;
+    MixSetupParms.ulFormatTag = MCI_WAVE_FORMAT_PCM;
+    MixSetupParms.ulSamplesPerSec = rate;
+    MixSetupParms.ulChannels = 2;
+    MixSetupParms.ulFormatMode = MCI_PLAY;
+    MixSetupParms.ulDeviceType = MCI_DEVTYPE_WAVEFORM_AUDIO;
+    MixSetupParms.pmixEvent = OS2_Dart_UpdateBuffers;
+
+    if (mciSendCommand(DeviceID, MCI_MIXSETUP,
+                       MCI_WAIT | MCI_MIXSETUP_INIT,
+                       (PVOID) & MixSetupParms, 0) != MCIERR_SUCCESS) {
+
+        mciSendCommand(DeviceID, MCI_CLOSE, MCI_WAIT,
+                       (PVOID) & GenericParms, 0);
+        fprintf(stderr, "Failed DART mixer setup\r\n");
+        return (-1);
+    }
+
+    /*bsize = MixSetupParms.ulBufferSize;*/
+    /*printf("Dart Buffer Size = %lu\n", bsize);*/
+
+    BufferParms.ulNumBuffers = BUFFERCOUNT;
+    BufferParms.ulBufferSize = bsize;
+    BufferParms.pBufList = MixBuffers;
+
+    if (mciSendCommand(DeviceID, MCI_BUFFER,
+                       MCI_WAIT | MCI_ALLOCATE_MEMORY,
+                       (PVOID) & BufferParms, 0) != MCIERR_SUCCESS) {
+        fprintf(stderr, "DART Memory allocation error\r\n");
+        mciSendCommand(DeviceID, MCI_CLOSE, MCI_WAIT,
+                       (PVOID) & GenericParms, 0);
+        return (-1);
+    }
+
+    for (i = 0; i < BUFFERCOUNT; i++) {
+        MixBuffers[i].ulBufferLength = bsize;
+    }
+
+    /* Start Playback */
+    memset(MixBuffers[0].pBuffer, /*32767 */ 0, bsize);
+    memset(MixBuffers[1].pBuffer, /*32767 */ 0, bsize);
+    MixSetupParms.pmixWrite(MixSetupParms.ulMixHandle, MixBuffers, 2);
+
+    send_output = write_dart_output;
+    close_output = close_dart_output;
+    pause_output = pause_output_nop;
+    resume_output = resume_output_nop;
+
+    return (0);
+}
+
+static int
+write_dart_output (int8_t *output_data, int output_size) {
+    static int idx = 0;
+
+    if (idx + output_size > bsize) {
+        do {
+            DosRequestMutexSem(dart_mutex, SEM_INDEFINITE_WAIT);
+            if (ready != 0) {
+                DosReleaseMutexSem(dart_mutex);
+                break;
+            }
+            DosReleaseMutexSem(dart_mutex);
+            DosSleep(20);
+        } while (TRUE);
+
+        MixBuffers[next].ulBufferLength = idx;
+        MixSetupParms.pmixWrite(MixSetupParms.ulMixHandle, &(MixBuffers[next]), 1);
+        ready--;
+        next++;
+        idx = 0;
+        if (next == BUFFERCOUNT) {
+            next = 0;
+        }
+    }
+    memcpy(&((char *)MixBuffers[next].pBuffer)[idx], output_data, output_size);
+    idx += output_size;
+    return (0);
+}
+
+static void
+close_dart_output (void) {
+    printf("Shutting down sound output\r\n");
+    if (MixBuffers[0].pBuffer) {
+        mciSendCommand(DeviceID, MCI_BUFFER,
+                       MCI_WAIT | MCI_DEALLOCATE_MEMORY, &BufferParms, 0);
+        MixBuffers[0].pBuffer = NULL;
+    }
+    if (DeviceID) {
+        mciSendCommand(DeviceID, MCI_CLOSE, MCI_WAIT,
+                       (PVOID) &GenericParms, 0);
+        DeviceID = 0;
+    }
+}
+
 #elif defined(__DJGPP__) && defined(AUDIODRV_DOSSB)
 /* SoundBlaster/Pro/16/AWE32 driver for DOS -- adapted from
  * libMikMod,  written by Andrew Zabolotny <bit@eltech.ru>,
@@ -693,6 +1019,120 @@ static int open_sb_output(void)
     return 0;
 }
 
+#elif defined(WILDMIDI_AMIGA) && defined(AUDIODRV_AHI)
+
+/* Driver for output to native Amiga AHI device:
+ * Written by Szilárd Biró <col.lawrence@gmail.com>, loosely based
+ * on an old AOS4 version by Fredrik Wikstrom <fredrik@a500.org>
+ */
+
+#define BUFFERSIZE (4 << 10)
+
+static struct MsgPort *AHImp = NULL;
+static struct AHIRequest *AHIReq[2] = { NULL, NULL };
+static int active = 0;
+static int8_t *AHIBuf[2] = { NULL, NULL };
+
+#define open_audio_output open_ahi_output
+static int write_ahi_output(int8_t *output_data, int output_size);
+static void close_ahi_output(void);
+
+static int open_ahi_output(void) {
+    AHImp = CreateMsgPort();
+    if (AHImp) {
+        AHIReq[0] = (struct AHIRequest *) CreateIORequest(AHImp, sizeof(struct AHIRequest));
+        if (AHIReq[0]) {
+            AHIReq[0]->ahir_Version = 4;
+            AHIReq[1] = (struct AHIRequest *) AllocVec(sizeof(struct AHIRequest), SHAREDMEMFLAG);
+            if (AHIReq[1]) {
+                if (!OpenDevice(AHINAME, AHI_DEFAULT_UNIT, (struct IORequest *)AHIReq[0], 0)) {
+                    /*AHIReq[0]->ahir_Std.io_Message.mn_Node.ln_Pri = 0;*/
+                    AHIReq[0]->ahir_Std.io_Command = CMD_WRITE;
+                    AHIReq[0]->ahir_Std.io_Data = NULL;
+                    AHIReq[0]->ahir_Std.io_Offset = 0;
+                    AHIReq[0]->ahir_Frequency = rate;
+                    AHIReq[0]->ahir_Type = AHIST_S16S;/* 16 bit stereo */
+                    AHIReq[0]->ahir_Volume = 0x10000;
+                    AHIReq[0]->ahir_Position = 0x8000;
+                    CopyMem(AHIReq[0], AHIReq[1], sizeof(struct AHIRequest));
+
+                    AHIBuf[0] = (int8_t *) AllocVec(BUFFERSIZE, SHAREDMEMFLAG | MEMF_CLEAR);
+                    if (AHIBuf[0]) {
+                        AHIBuf[1] = (int8_t *) AllocVec(BUFFERSIZE, SHAREDMEMFLAG | MEMF_CLEAR);
+                        if (AHIBuf[1]) {
+                            send_output = write_ahi_output;
+                            close_output = close_ahi_output;
+                            pause_output = pause_output_nop;
+                            resume_output = resume_output_nop;
+                            return (0);
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    close_ahi_output();
+    fprintf(stderr, "ERROR: Unable to open AHI output\r\n");
+    return (-1);
+}
+
+static int write_ahi_output(int8_t *output_data, int output_size) {
+    int chunk;
+    while (output_size > 0) {
+        if (AHIReq[active]->ahir_Std.io_Data) {
+            WaitIO((struct IORequest *) AHIReq[active]);
+        }
+        chunk = (output_size < BUFFERSIZE)? output_size : BUFFERSIZE;
+        memcpy(AHIBuf[active], output_data, chunk);
+        output_size -= chunk;
+        output_data += chunk;
+
+        AHIReq[active]->ahir_Std.io_Data = AHIBuf[active];
+        AHIReq[active]->ahir_Std.io_Length = chunk;
+        AHIReq[active]->ahir_Link = !CheckIO((struct IORequest *) AHIReq[active ^ 1]) ? AHIReq[active ^ 1] : NULL;
+        SendIO((struct IORequest *)AHIReq[active]);
+        active ^= 1;
+    }
+    return (0);
+}
+
+static void close_ahi_output(void) {
+    if (AHIReq[1]) {
+        AHIReq[0]->ahir_Link = NULL; /* in case we are linked to req[0] */
+        if (!CheckIO((struct IORequest *) AHIReq[1])) {
+            AbortIO((struct IORequest *) AHIReq[1]);
+            WaitIO((struct IORequest *) AHIReq[1]);
+        }
+        FreeVec(AHIReq[1]);
+        AHIReq[1] = NULL;
+    }
+    if (AHIReq[0]) {
+        if (!CheckIO((struct IORequest *) AHIReq[0])) {
+            AbortIO((struct IORequest *) AHIReq[0]);
+            WaitIO((struct IORequest *) AHIReq[0]);
+        }
+        if (AHIReq[0]->ahir_Std.io_Device) {
+            CloseDevice((struct IORequest *) AHIReq[0]);
+            AHIReq[0]->ahir_Std.io_Device = NULL;
+        }
+        DeleteIORequest((struct IORequest *) AHIReq[0]);
+        AHIReq[0] = NULL;
+    }
+    if (AHImp) {
+        DeleteMsgPort(AHImp);
+        AHImp = NULL;
+    }
+    if (AHIBuf[0]) {
+        FreeVec(AHIBuf[0]);
+        AHIBuf[0] = NULL;
+    }
+    if (AHIBuf[1]) {
+        FreeVec(AHIBuf[1]);
+        AHIBuf[1] = NULL;
+    }
+}
+
 #else
 #ifdef AUDIODRV_ALSA
 
@@ -1049,6 +1489,7 @@ static int open_openal_output(void) {
 
 #define open_audio_output open_noaudio_output
 static int open_noaudio_output(void) {
+    fprintf(stderr, "No audio output driver was selected at compile time.\r\n");
     return -1;
 }
 
@@ -1139,31 +1580,30 @@ int main(int argc, char **argv) {
     uint32_t apr_mins;
     uint32_t apr_secs;
     char modes[5];
-//    uint32_t count_diff = 0;
     uint8_t ch;
-    uint8_t test_midi = 0;
-    uint8_t test_count = 0;
+    int test_midi = 0;
+    int test_count = 0;
     uint8_t *test_data;
     uint8_t test_bank = 0;
     uint8_t test_patch = 0;
     static char spinner[] = "|/-\\";
     static int spinpoint = 0;
     unsigned long int seek_to_sample;
-    unsigned long int samples = 0;
+    uint32_t samples = 0;
     int inpause = 0;
     char * ret_err = NULL;
     long libraryver;
     char * lyric = NULL;
     char *last_lyric = NULL;
-    uint32_t last_lyric_length = 0;
+    size_t last_lyric_length = 0;
     int8_t kareoke = 0;
 #define MAX_LYRIC_CHAR 128
     char lyrics[MAX_LYRIC_CHAR + 1];
 #define MAX_DISPLAY_LYRICS 29
     char display_lyrics[MAX_DISPLAY_LYRICS + 1];
-    
-    unsigned long int play_from = 0.0;
-    unsigned long int play_to = 0.0;
+
+    unsigned long int play_from = 0;
+    unsigned long int play_to = 0;
 
     memset(lyrics,' ',MAX_LYRIC_CHAR);
     memset(display_lyrics,' ',MAX_DISPLAY_LYRICS);
@@ -1194,7 +1634,7 @@ int main(int argc, char **argv) {
                 fprintf(stderr, "Error: bad rate %i.\n", res);
                 return (1);
             }
-            rate = res;
+            rate = (uint32_t) res;
             break;
         case 'b': /* Reverb */
             mixer_options |= WM_MO_REVERB;
@@ -1211,10 +1651,10 @@ int main(int argc, char **argv) {
             wav_file[sizeof(wav_file) - 1] = 0;
             break;
         case 'g': /* XMIDI Conversion */
-            WildMidi_SetCvtOption(WM_CO_XMI_TYPE, atoi(optarg));
+            WildMidi_SetCvtOption(WM_CO_XMI_TYPE, (uint16_t) atoi(optarg));
             break;
         case 'f': /* MIDI-like Conversion */
-            WildMidi_SetCvtOption(WM_CO_FREQUENCY, atoi(optarg));
+            WildMidi_SetCvtOption(WM_CO_FREQUENCY, (uint16_t) atoi(optarg));
             break;
         case 'x': /* MIDI Output */
             if (!*optarg) {
@@ -1307,7 +1747,8 @@ int main(int argc, char **argv) {
 
         printf("Converting %s\r\n", real_file);
         if (WildMidi_ConvertToMidi(argv[optind], &data, &size) < 0) {
-            fprintf(stderr, "Conversion failed.\r\n");
+            fprintf(stderr, "Conversion failed: %s.\r\n", WildMidi_GetError());
+            WildMidi_ClearError();
             return (1);
         }
 
@@ -1339,6 +1780,8 @@ int main(int argc, char **argv) {
                         (libraryver>> 8) & 255,
                         (libraryver    ) & 255);
     if (WildMidi_Init(config_file, rate, mixer_options) == -1) {
+        fprintf(stderr, "%s\r\n", WildMidi_GetError());
+        WildMidi_ClearError();
         return (1);
     }
 
@@ -1347,7 +1790,7 @@ int main(int argc, char **argv) {
     printf(" ,  1sec Seek Back   r  Reverb               .  1sec Seek Forward\n");
     printf(" m  save as midi     p  Pause On/Off\n\n");
 
-    output_buffer = malloc(16384);
+    output_buffer = (int8_t *) malloc(16384);
     if (output_buffer == NULL) {
         fprintf(stderr, "Not enough memory, exiting\n");
         WildMidi_Shutdown();
@@ -1355,6 +1798,9 @@ int main(int argc, char **argv) {
     }
 
     wm_inittty();
+#ifdef WILDMIDI_AMIGA
+    amiga_sysinit();
+#endif
 
     WildMidi_MasterVolume(master_volume);
 
@@ -1378,7 +1824,7 @@ int main(int argc, char **argv) {
             if (test_count == midi_test_max) {
                 break;
             }
-            test_data = malloc(midi_test[test_count].size);
+            test_data = (uint8_t *) malloc(midi_test[test_count].size);
             memcpy(test_data, midi_test[test_count].data,
                     midi_test[test_count].size);
             test_data[25] = test_bank;
@@ -1408,33 +1854,29 @@ int main(int argc, char **argv) {
 
         memset(lyrics,' ',MAX_LYRIC_CHAR);
         memset(display_lyrics,' ',MAX_DISPLAY_LYRICS);
-        
+
         if (play_from != 0) {
             WildMidi_FastSeek(midi_ptr, &play_from);
             if (play_to < play_from) {
                 // Ignore --playto if set less than --playfrom
-            }   play_to = 0;
+                play_to = 0;
+            }
         }
 
         while (1) {
-#if 0
-            count_diff = wm_info->approx_total_samples
-                        - wm_info->current_sample;
-
-            if (count_diff == 0)
-                break;
-#endif
             ch = 0;
 #ifdef _WIN32
             if (_kbhit()) {
                 ch = _getch();
                 _putch(ch);
             }
-#elif defined(__DJGPP__)
+#elif defined(__DJGPP__) || defined(__OS2__) || defined(__EMX__)
             if (kbhit()) {
                 ch = getch();
                 putch(ch);
             }
+#elif defined(WILDMIDI_AMIGA)
+            amiga_getch (&ch);
 #else
             if (read(STDIN_FILENO, &ch, 1) != 1)
                 ch = 0;
@@ -1564,29 +2006,28 @@ int main(int argc, char **argv) {
                 pro_secs = (wm_info->current_sample % (rate * 60)) / rate;
                 fprintf(stderr,
                         "%s [%s] [%3i] [%2um %2us Processed] [%2u%%] P  \r",
-                        display_lyrics, modes, master_volume, pro_mins,
+                        display_lyrics, modes, (int)master_volume, pro_mins,
                         pro_secs, perc_play);
                 msleep(5);
                 continue;
             }
 
-            
             if (play_to != 0) {
                 if ((wm_info->current_sample + 4096) <= play_to) {
                     samples = 16384;
                 } else {
                     samples = (play_to - wm_info->current_sample) << 2;
-                    if (samples <= 0) {
+                    if (!samples) {
                         // We are at or past where we wanted to play to
                         break;
                     }
-                    
                 }
-            } else {
+            }
+            else {
                 samples = 16384;
             }
             res = WildMidi_GetOutput(midi_ptr, output_buffer, samples);
-//                                     (count_diff >= 4096)? 16384 : (count_diff * 4));
+
             if (res <= 0)
                 break;
 
@@ -1616,7 +2057,7 @@ int main(int argc, char **argv) {
             pro_secs = (wm_info->current_sample % (rate * 60)) / rate;
             fprintf(stderr,
                 "%s [%s] [%3i] [%2um %2us Processed] [%2u%%] %c  \r",
-                display_lyrics, modes, master_volume, pro_mins,
+                display_lyrics, modes, (int)master_volume, pro_mins,
                 pro_secs, perc_play, spinner[spinpoint++ % 4]);
 
             if (send_output(output_buffer, res) < 0) {
@@ -1636,10 +2077,6 @@ int main(int argc, char **argv) {
 end1: memset(output_buffer, 0, 16384);
     send_output(output_buffer, 16384);
     msleep(5);
-#if 0/*#ifdef AUDIODRV_OPENAL*/
-    /* FIXME: Delay needed in OPENAL before exiting to complete the song. */
-    msleep(1000);
-#endif
 end2: close_output();
     free(output_buffer);
     if (WildMidi_Shutdown() == -1) {
@@ -1655,7 +2092,7 @@ end2: close_output();
 
 /* helper / replacement functions: */
 
-#if !defined(_WIN32) && !defined(__DJGPP__)
+#if !(defined(_WIN32) || defined(__DJGPP__) || defined(WILDMIDI_AMIGA) || defined(__OS2__) || defined(__EMX__))
 static int msleep(unsigned long milisec) {
     struct timespec req = { 0, 0 };
     time_t sec = (int) (milisec / 1000);
