@@ -262,7 +262,7 @@ static inline int wm_isupper(int c) {
 static inline int wm_tolower(int c) {
     return ((wm_isupper(c)) ? (c | ('a' - 'A')) : c);
 }
-#if 0 /* clang whines that these aren't used. */
+#if 0 /* not used for now. */
 static inline int wm_islower(int c) {
     return (c >= 'a' && c <= 'z');
 }
@@ -330,7 +330,7 @@ static char** WM_LC_Tokenize_Line(char *line_data) {
                     token_data_length += TOKEN_CNT_INC;
                     token_data = (char **) realloc(token_data, token_data_length * sizeof(char *));
                     if (token_data == NULL) {
-                        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM,"to parse config", errno);
+                        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
                         return (NULL);
                     }
                 }
@@ -361,6 +361,7 @@ static int load_config(const char *config_file, const char *conf_dir) {
     uint32_t config_ptr = 0;
     uint32_t line_start_ptr = 0;
     uint16_t patchid = 0;
+    const char *dotpat;
     struct _patch * tmp_patch;
     char **line_tokens = NULL;
     int token_count = 0;
@@ -373,9 +374,9 @@ static int load_config(const char *config_file, const char *conf_dir) {
 
     if (conf_dir) {
         if (!(config_dir = wm_strdup(conf_dir))) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, config_file, errno);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
             WM_FreePatches();
-            free(config_buffer);
+            _WM_FreeBufferFile(config_buffer);
             return (-1);
         }
     } else {
@@ -383,7 +384,7 @@ static int load_config(const char *config_file, const char *conf_dir) {
         if (dir_end) {
             config_dir = (char *) malloc((dir_end - config_file + 2));
             if (config_dir == NULL) {
-                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, config_file, errno);
+                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
                 WM_FreePatches();
                 free(config_buffer);
                 return (-1);
@@ -416,13 +417,13 @@ static int load_config(const char *config_file, const char *conf_dir) {
                             _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID_ARG, "(missing name in dir line)", 0);
                             WM_FreePatches();
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             return (-1);
                         } else if (!(config_dir = wm_strdup(line_tokens[1]))) {
-                            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, config_file, errno);
+                            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
                             WM_FreePatches();
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             return (-1);
                         }
                         if (!IS_DIR_SEPARATOR(config_dir[strlen(config_dir) - 1])) {
@@ -435,33 +436,33 @@ static int load_config(const char *config_file, const char *conf_dir) {
                             _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID_ARG, "(missing name in source line)", 0);
                             WM_FreePatches();
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             return (-1);
                         } else if (!IS_ABSOLUTE_PATH(line_tokens[1]) && config_dir) {
                             new_config = (char *) malloc(strlen(config_dir) + strlen(line_tokens[1]) + 1);
                             if (new_config == NULL) {
-                                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, config_file, errno);
+                                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
                                 WM_FreePatches();
                                 free(config_dir);
                                 free(line_tokens);
-                                free(config_buffer);
+                                _WM_FreeBufferFile(config_buffer);
                                 return (-1);
                             }
                             strcpy(new_config, config_dir);
                             strcpy(&new_config[strlen(config_dir)], line_tokens[1]);
                         } else {
                             if (!(new_config = wm_strdup(line_tokens[1]))) {
-                                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, config_file, errno);
+                                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
                                 WM_FreePatches();
                                 free(line_tokens);
-                                free(config_buffer);
+                                _WM_FreeBufferFile(config_buffer);
                                 return (-1);
                             }
                         }
                         if (load_config(new_config, config_dir) == -1) {
                             free(new_config);
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             free(config_dir);
                             return (-1);
                         }
@@ -472,7 +473,7 @@ static int load_config(const char *config_file, const char *conf_dir) {
                             WM_FreePatches();
                             free(config_dir);
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             return (-1);
                         }
                         patchid = (atoi(line_tokens[1]) & 0xFF) << 8;
@@ -482,7 +483,7 @@ static int load_config(const char *config_file, const char *conf_dir) {
                             WM_FreePatches();
                             free(config_dir);
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             return (-1);
                         }
                         patchid = ((atoi(line_tokens[1]) & 0xFF) << 8) | 0x80;
@@ -492,15 +493,15 @@ static int load_config(const char *config_file, const char *conf_dir) {
                             WM_FreePatches();
                             free(config_dir);
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             return (-1);
                         }
                         _WM_reverb_room_width = (float) atof(line_tokens[1]);
                         if (_WM_reverb_room_width < 1.0f) {
-                            _WM_DEBUG_MSG("%s: reverb_room_width < 1 meter, setting to minimum of 1 meter", config_file);
+                            _WM_DEBUG_MSG("%s: reverb_room_width < 1m, setting to 1m", config_file);
                             _WM_reverb_room_width = 1.0f;
                         } else if (_WM_reverb_room_width > 100.0f) {
-                            _WM_DEBUG_MSG("%s: reverb_room_width > 100 meters, setting to maximum of 100 meters", config_file);
+                            _WM_DEBUG_MSG("%s: reverb_room_width > 100m, setting to 100m", config_file);
                             _WM_reverb_room_width = 100.0f;
                         }
                     } else if (wm_strcasecmp(line_tokens[0], "reverb_room_length") == 0) {
@@ -509,15 +510,15 @@ static int load_config(const char *config_file, const char *conf_dir) {
                             WM_FreePatches();
                             free(config_dir);
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             return (-1);
                         }
                         _WM_reverb_room_length = (float) atof(line_tokens[1]);
                         if (_WM_reverb_room_length < 1.0f) {
-                            _WM_DEBUG_MSG("%s: reverb_room_length < 1 meter, setting to minimum of 1 meter", config_file);
+                            _WM_DEBUG_MSG("%s: reverb_room_length < 1m, setting to 1m", config_file);
                             _WM_reverb_room_length = 1.0f;
                         } else if (_WM_reverb_room_length > 100.0f) {
-                            _WM_DEBUG_MSG("%s: reverb_room_length > 100 meters, setting to maximum of 100 meters", config_file);
+                            _WM_DEBUG_MSG("%s: reverb_room_length > 100m, setting to 100m", config_file);
                             _WM_reverb_room_length = 100.0f;
                         }
                     } else if (wm_strcasecmp(line_tokens[0], "reverb_listener_posx") == 0) {
@@ -526,7 +527,7 @@ static int load_config(const char *config_file, const char *conf_dir) {
                             WM_FreePatches();
                             free(config_dir);
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             return (-1);
                         }
                         _WM_reverb_listen_posx = (float) atof(line_tokens[1]);
@@ -542,7 +543,7 @@ static int load_config(const char *config_file, const char *conf_dir) {
                             WM_FreePatches();
                             free(config_dir);
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             return (-1);
                         }
                         _WM_reverb_listen_posy = (float) atof(line_tokens[1]);
@@ -564,11 +565,11 @@ static int load_config(const char *config_file, const char *conf_dir) {
                         if (_WM_patch[(patchid & 0x7F)] == NULL) {
                             _WM_patch[(patchid & 0x7F)] = (struct _patch *) malloc(sizeof(struct _patch));
                             if (_WM_patch[(patchid & 0x7F)] == NULL) {
-                                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, config_file, errno);
+                                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
                                 WM_FreePatches();
                                 free(config_dir);
                                 free(line_tokens);
-                                free(config_buffer);
+                                _WM_FreeBufferFile(config_buffer);
                                 return (-1);
                             }
                             tmp_patch = _WM_patch[(patchid & 0x7F)];
@@ -596,11 +597,11 @@ static int load_config(const char *config_file, const char *conf_dir) {
                                     }
                                     if (tmp_patch->next == NULL) {
                                         if ((tmp_patch->next = (struct _patch *) malloc(sizeof(struct _patch))) == NULL) {
-                                            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, config_file, 0);
+                                            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, 0);
                                             WM_FreePatches();
                                             free(config_dir);
                                             free(line_tokens);
-                                            free(config_buffer);
+                                            _WM_FreeBufferFile(config_buffer);
                                             return (-1);
                                         }
                                         tmp_patch = tmp_patch->next;
@@ -622,11 +623,11 @@ static int load_config(const char *config_file, const char *conf_dir) {
                                 } else {
                                     tmp_patch->next = (struct _patch *) malloc(sizeof(struct _patch));
                                     if (tmp_patch->next == NULL) {
-                                        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, config_file, errno);
+                                        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
                                         WM_FreePatches();
                                         free(config_dir);
                                         free(line_tokens);
-                                        free(config_buffer);
+                                        _WM_FreeBufferFile(config_buffer);
                                         return (-1);
                                     }
                                     tmp_patch = tmp_patch->next;
@@ -646,31 +647,32 @@ static int load_config(const char *config_file, const char *conf_dir) {
                             WM_FreePatches();
                             free(config_dir);
                             free(line_tokens);
-                            free(config_buffer);
+                            _WM_FreeBufferFile(config_buffer);
                             return (-1);
                         } else if (!IS_ABSOLUTE_PATH(line_tokens[1]) && config_dir) {
                             tmp_patch->filename = (char *) malloc(strlen(config_dir) + strlen(line_tokens[1]) + 5);
                             if (tmp_patch->filename == NULL) {
-                                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, config_file, 0);
+                                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, 0);
                                 WM_FreePatches();
                                 free(config_dir);
                                 free(line_tokens);
-                                free(config_buffer);
+                                _WM_FreeBufferFile(config_buffer);
                                 return (-1);
                             }
                             strcpy(tmp_patch->filename, config_dir);
                             strcat(tmp_patch->filename, line_tokens[1]);
                         } else {
                             if (!(tmp_patch->filename = wm_strdup(line_tokens[1]))) {
-                                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, config_file, 0);
+                                _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, 0);
                                 WM_FreePatches();
                                 free(config_dir);
                                 free(line_tokens);
-                                free(config_buffer);
+                                _WM_FreeBufferFile(config_buffer);
                                 return (-1);
                             }
                         }
-                        if (wm_strncasecmp(&tmp_patch->filename[strlen(tmp_patch->filename) - 4], ".pat", 4) != 0) {
+                        dotpat = strrchr(tmp_patch->filename, '.');
+                        if (!dotpat || wm_strcasecmp(dotpat, ".pat") != 0) {
                             strcat(tmp_patch->filename, ".pat");
                         }
                         tmp_patch->env[0].set = 0x00;
@@ -752,7 +754,7 @@ static int load_config(const char *config_file, const char *conf_dir) {
                 else if (_WM_Global_ErrorI) { /* malloc() failure in WM_LC_Tokenize_Line() */
                     WM_FreePatches();
                     free(line_tokens);
-                    free(config_buffer);
+                    _WM_FreeBufferFile(config_buffer);
                     return (-1);
                 }
                 /* free up tokens */
@@ -763,7 +765,7 @@ static int load_config(const char *config_file, const char *conf_dir) {
         config_ptr++;
     }
 
-    free(config_buffer);
+    _WM_FreeBufferFile(config_buffer);
     free(config_dir);
 
     return (0);
@@ -779,7 +781,7 @@ static int add_handle(void * handle) {
     if (first_handle == NULL) {
         first_handle = (struct _hndl *) malloc(sizeof(struct _hndl));
         if (first_handle == NULL) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, " to get ram", errno);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
             return (-1);
         }
         first_handle->handle = handle;
@@ -793,7 +795,7 @@ static int add_handle(void * handle) {
         }
         tmp_handle->next = (struct _hndl *) malloc(sizeof(struct _hndl));
         if (tmp_handle->next == NULL) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, " to get ram", errno);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, errno);
             return (-1);
         }
         tmp_handle->next->prev = tmp_handle;
@@ -819,7 +821,7 @@ static int add_handle(void * handle) {
 
 static int WM_GetOutput_Linear(midi * handle, int8_t *buffer, uint32_t size) {
     uint32_t buffer_used = 0;
-    uint32_t i;
+    uint32_t i, env_ptr;
     struct _mdi *mdi = (struct _mdi *) handle;
     uint32_t real_samples_to_mix = 0;
     uint32_t data_pos;
@@ -854,7 +856,7 @@ static int WM_GetOutput_Linear(midi * handle, int8_t *buffer, uint32_t size) {
         if (__builtin_expect((!mdi->samples_to_mix), 0)) {
             while ((!mdi->samples_to_mix) && (event->do_event)) {
                 event->do_event(mdi, &event->event_data);
-                if ((mdi->extra_info.mixer_options & WM_MO_LOOP) && (event[0].do_event == *_WM_do_meta_endoftrack)) {
+                if ((mdi->extra_info.mixer_options & WM_MO_LOOP) && (event[0].evtype == ev_meta_endoftrack)) {
                     _WM_ResetToStart(mdi);
                     event = mdi->current_event;
                 } else {
@@ -990,15 +992,16 @@ static int WM_GetOutput_Linear(midi * handle, int8_t *buffer, uint32_t size) {
                             note_data = note_data->next;
                             RESAMPLE_DEBUGS("Next Note: SAMPLE_SUSTAIN");
                             continue;
-                        } else if (note_data->modes & SAMPLE_CLAMPED) {
-                            note_data->env = 5;
+                        } else {
+                            env_ptr = (note_data->modes & SAMPLE_CLAMPED)? 5 : 4;
+                            note_data->env = env_ptr;
                             if (note_data->env_level
-                                    > note_data->sample->env_target[5]) {
+                                    > note_data->sample->env_target[env_ptr]) {
                                 note_data->env_inc =
-                                        -note_data->sample->env_rate[5];
+                                        -note_data->sample->env_rate[env_ptr];
                             } else {
                                 note_data->env_inc =
-                                        note_data->sample->env_rate[5];
+                                        note_data->sample->env_rate[env_ptr];
                             }
                             continue;
                         }
@@ -1133,7 +1136,7 @@ static int WM_GetOutput_Linear(midi * handle, int8_t *buffer, uint32_t size) {
 
 static int WM_GetOutput_Gauss(midi * handle, int8_t *buffer, uint32_t size) {
     uint32_t buffer_used = 0;
-    uint32_t i;
+    uint32_t i, env_ptr;
     struct _mdi *mdi = (struct _mdi *) handle;
     uint32_t real_samples_to_mix = 0;
     uint32_t data_pos;
@@ -1172,7 +1175,7 @@ static int WM_GetOutput_Gauss(midi * handle, int8_t *buffer, uint32_t size) {
         if (__builtin_expect((!mdi->samples_to_mix), 0)) {
             while ((!mdi->samples_to_mix) && (event->do_event)) {
                 event->do_event(mdi, &event->event_data);
-                if ((mdi->extra_info.mixer_options & WM_MO_LOOP) && (event[0].do_event == *_WM_do_meta_endoftrack)) {
+                if ((mdi->extra_info.mixer_options & WM_MO_LOOP) && (event[0].evtype == ev_meta_endoftrack)) {
                     _WM_ResetToStart(mdi);
                     event = mdi->current_event;
                 } else {
@@ -1335,15 +1338,16 @@ static int WM_GetOutput_Gauss(midi * handle, int8_t *buffer, uint32_t size) {
                                 note_data->env_inc = 0;
                                 note_data = note_data->next;
                                 continue;
-                            } else if (note_data->modes & SAMPLE_CLAMPED) {
-                                note_data->env = 5;
+                            } else {
+                                env_ptr = (note_data->modes & SAMPLE_CLAMPED)? 5 : 4;
+                                note_data->env = env_ptr;
                                 if (note_data->env_level
-                                    > note_data->sample->env_target[5]) {
+                                        > note_data->sample->env_target[env_ptr]) {
                                     note_data->env_inc =
-                                    -note_data->sample->env_rate[5];
+                                        -note_data->sample->env_rate[env_ptr];
                                 } else {
                                     note_data->env_inc =
-                                    note_data->sample->env_rate[5];
+                                        note_data->sample->env_rate[env_ptr];
                                 }
                                 continue;
                             }
@@ -1485,7 +1489,7 @@ WM_SYMBOL int WildMidi_ConvertToMidi (const char *file, uint8_t **out, uint32_t
     }
 
     ret = WildMidi_ConvertBufferToMidi(buf, *size, out, size);
-    free(buf);
+    _WM_FreeBufferFile(buf);
     return ret;
 }
 
@@ -1533,7 +1537,8 @@ WM_SYMBOL long WildMidi_GetVersion (void) {
     return (LIBWILDMIDI_VERSION);
 }
 
-WM_SYMBOL int WildMidi_Init(const char *config_file, uint16_t rate, uint16_t mixer_options) {
+static int _WM_Init(const struct _WM_VIO *callbacks,
+                    const char *config_file, uint16_t rate, uint16_t mixer_options) {
     if (WM_Initialized) {
         _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_ALR_INIT, NULL, 0);
         return (-1);
@@ -1544,6 +1549,10 @@ WM_SYMBOL int WildMidi_Init(const char *config_file, uint16_t rate, uint16_t mix
                 "(NULL config file pointer)", 0);
         return (-1);
     }
+
+    _WM_BufferFile = callbacks->allocate_file;
+    _WM_FreeBufferFile = callbacks->free_file;
+
     WM_InitPatches();
     if (WM_LoadConfig(config_file) == -1) {
         return (-1);
@@ -1573,6 +1582,20 @@ WM_SYMBOL int WildMidi_Init(const char *config_file, uint16_t rate, uint16_t mix
     return (0);
 }
 
+WM_SYMBOL int WildMidi_Init(const char *config_file, uint16_t rate, uint16_t mixer_options) {
+    struct _WM_VIO callbacks_ = { _WM_BufferFileImpl, _WM_FreeBufferFileImpl };
+    return _WM_Init(&callbacks_, config_file, rate, mixer_options);
+}
+
+WM_SYMBOL int WildMidi_InitVIO(struct _WM_VIO *callbacks, const char *config_file, uint16_t rate, uint16_t mixer_options) {
+    if (!callbacks || !callbacks->allocate_file || !callbacks->free_file) {
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID_ARG, "(NULL VIO callbacks)", 0);
+        return (-1);
+    }
+
+    return _WM_Init(callbacks, config_file, rate, mixer_options);
+}
+
 WM_SYMBOL int WildMidi_MasterVolume(uint8_t master_volume) {
     if (!WM_Initialized) {
         _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_NOT_INIT, NULL, 0);
@@ -1668,7 +1691,7 @@ WM_SYMBOL midi *WildMidi_Open(const char *midifile) {
     } else {
         ret = (void *) _WM_ParseNewMidi(mididata, midisize);
     }
-    free(mididata);
+    _WM_FreeBufferFile(mididata);
 
     if (ret) {
         if (add_handle(ret) != 0) {
@@ -1839,7 +1862,7 @@ WM_SYMBOL int WildMidi_SongSeek (midi * handle, int8_t nextsong) {
         return (-1);
     }
     if ((nextsong > 1) || (nextsong < -1)) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID_ARG, "(Invalid nextsong setting. -1 is previous song, 0 start of current song, 1 is next song)", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID_ARG, "(Invalid nextsong: -1 is previous song, 0 is start of current song, 1 is next song)", 0);
         _WM_Unlock(&mdi->lock);
         return (-1);
     }
@@ -1857,7 +1880,7 @@ WM_SYMBOL int WildMidi_SongSeek (midi * handle, int8_t nextsong) {
          */
         uint8_t eof_cnt = 1;
         while (event != mdi->events) {
-            if (event[-1].do_event == _WM_do_meta_endoftrack) {
+            if (event[-1].evtype == ev_meta_endoftrack) {
                 if (eof_cnt == 0) {
                     break;
                 }
@@ -1871,10 +1894,10 @@ WM_SYMBOL int WildMidi_SongSeek (midi * handle, int8_t nextsong) {
 
     } else if (nextsong == 1) {
         /* goto start of next song */
-        while (event->do_event != NULL) {
-            if (event->do_event == _WM_do_meta_endoftrack) {
+        while (event->evtype != ev_null) {
+            if (event->evtype == ev_meta_endoftrack) {
                 event++;
-                if (event->do_event == NULL) {
+                if (event->evtype == ev_null) {
                     event--;
                     goto START_THIS_SONG;
                 } else {
@@ -1891,7 +1914,7 @@ WM_SYMBOL int WildMidi_SongSeek (midi * handle, int8_t nextsong) {
         /* goto start of this song */
         /* first find the offset */
         while (event != mdi->events) {
-            if (event[-1].do_event == _WM_do_meta_endoftrack) {
+            if (event[-1].evtype == ev_meta_endoftrack) {
                 break;
             }
             event--;
@@ -2042,7 +2065,7 @@ WildMidi_GetInfo(midi * handle) {
     if (mdi->tmp_info == NULL) {
         mdi->tmp_info = (struct _WM_Info *) malloc(sizeof(struct _WM_Info));
         if (mdi->tmp_info == NULL) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to set info", 0);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, 0);
             _WM_Unlock(&mdi->lock);
             return (NULL);
         }
@@ -2058,7 +2081,7 @@ WildMidi_GetInfo(midi * handle) {
         if (mdi->tmp_info->copyright == NULL) {
             free(mdi->tmp_info);
             mdi->tmp_info = NULL;
-            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to set copyright", 0);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, NULL, 0);
             _WM_Unlock(&mdi->lock);
             return (NULL);
         } else {
@@ -2099,6 +2122,9 @@ WM_SYMBOL int WildMidi_Shutdown(void) {
 
     if (_WM_Global_ErrorS != NULL) free(_WM_Global_ErrorS);
 
+    _WM_BufferFile = _WM_BufferFileImpl;
+    _WM_FreeBufferFile = _WM_FreeBufferFileImpl;
+
     return (0);
 }
 
